from django.contrib import admin
from .models import CIE10, Source


@admin.register(CIE10)
class CIE10Admin(admin.ModelAdmin):
    list_display = ['code', 'source', 'description', 'level', 'depends_on']
    search_fields = ['code', 'description']
    list_filter = ['source', 'depends_on']


@admin.register(Source)
class SourceAdmin(admin.ModelAdmin):
    list_display = ['name', 'description']



from django.db import models
import logging
logger = logging.getLogger(__name__)


class Source(models.Model):
    name = models.CharField(max_length=40)
    description = models.TextField(null=True, blank=True)

    def __str__(self):
        return self.name


class CIE10(models.Model):
    code = models.CharField(max_length=10, unique=True)
    description = models.TextField(null=True, blank=True)
    depends_on = models.ForeignKey('self', null=True, blank=True, on_delete=models.CASCADE, related_name='children')
    level = models.IntegerField(default=0)
    source = models.ForeignKey(Source, null=True, blank=True, on_delete=models.SET_NULL, related_name='codes')

    def __str__(self):
        return self.code

    @classmethod
    def start_db(cls):
        logger.info('Comenzando importacion de códigos CIE10')
        from cie.cie10 import CIECodes
        cie = CIECodes()
        c = 0
        for code, content in cie.tree.items():
            c += 1
            logger.info(f'{c} Imporatando código {code}')
            source, created = Source.objects.get_or_create(name=content['source'])
            c10, created = CIE10.objects.get_or_create(code=code)
            c10.description = content['description']
            c10.source = source
            c10.level = content['level']
            c10.save()
        
        for code, content in cie.tree.items():
            logger.info(f'Vinculando código {code}')
            c10 = CIE10.objects.get(code=code)
            if c10.level > 0:
                depon_code = content.get('depends_on', '')    
                depon = CIE10.objects.get(code=depon_code)
                c10.depends_on = depon
                c10.save()
        
        return c


""" CIE dict """
import csv
import json
import os
import zipfile


class CIECodes:
    here = os.path.dirname(os.path.realpath(__file__))
    data_folder = os.path.join(here, 'data')
    local_zip_csv = os.path.join(data_folder, 'cie-10.zip')
    local_csv = os.path.join(data_folder, 'cie-10.csv')  # robado de acá https://github.com/verasativa/CIE-10
    local_json = os.path.join(data_folder, 'cie-10.json')

    tree = {}

    def __init__(self):
        if self.tree == {}:
            self.read_json()

    def info(self, code):
        if code not in self.tree:
            code = code.replace('.', '')  # W231 == W23.1
            if code not in self.tree:
                return None
        
        ret = self.tree[code]
        descriptions = [ret['description']]
        while ret['depends_on'] is not None:
            ret = self.tree[ret['depends_on']]
            descriptions.append(ret['description'])

        ret = self.tree[code]
        ret['multiple_descriptions'] = descriptions
        ret['code'] = code
        return ret

    def read_json(self):
        # read defined JSON an load to the _self.tree_ property
        if os.path.isfile(self.local_json):
            f = open(self.local_json, 'r')
            self.tree = json.load(f)
            f.close
        else:
            self.read_csv()

    def read_csv(self):
        # read CSV and transfor to a useful JSON
        # extracts CSV if is not ready
        if not os.path.isfile(self.local_csv):
            self.extract_csv()
            
        tree = {}   # results
        fieldnames = ['code', 'code_0', 'code_1', 'code_2', 'code_3',
                      'code_4', 'description', 'level', 'source']
        f = open(self.local_csv, 'r')
        reader = csv.DictReader(f, fieldnames=fieldnames)
        next(f)  # headers
        errors = []
        for row in reader:
            # fix all shit
            for k, v in row.items():
                row[k] = v.strip().replace('–', '-')

            code = row['code']
            
            if code in self.tree:
                raise Exception('Duplicated code {}'.format(code))
            
            for coden in ['code_0', 'code_1', 'code_2', 'code_3', 'code_4']:
                rcoden = row[coden]
                if rcoden != '' and rcoden not in tree:
                    raise Exception('Missing code {}'.format(rcoden))
            
            level = int(row['level'])
            tree[code] = {'description': row['description'],
                          'source': row['source'],
                          'level': level} 
            
            depends_on_field = 'code_{}'.format(level-1)
            
            if level > 0:
                if row[depends_on_field] not in tree:
                    raise Exception('Parent node do not exists {}'.format(row[depends_on_field]))
                tree[code]['depends_on'] = row[depends_on_field]
            else:
                tree[code]['depends_on'] = None

        f.close()

        f2 = open(self.local_json, 'w')
        f2.write(json.dumps(tree, indent=2))
        f2.close

        self.tree = tree
    
    def extract_csv(self):
        with zipfile.ZipFile(self.local_zip_csv, 'r') as zip_ref:
            zip_ref.extractall(self.data_folder)
    
    def search(self, txt):
        for code, content in self.tree.items():
            full_info = self.info(code=code)
            # saca del medio, Solr
            full_str = ' '.join([str(val).lower() for key, val in full_info.items()])
            if full_str.find(txt.lower()) > -1:
                yield full_info




# Generated by Django 2.2.4 on 2019-09-21 16:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Source',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=40)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='CIE10',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=10, unique=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('level', models.IntegerField(default=0)),
                ('depends_on', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='cie10_django.CIE10')),
                ('source', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='codes', to='cie10_django.Source')),
            ],
        )
    ]



import setuptools

print(setuptools.find_packages())

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='cie',
    version='0.207',
    license='MIT',
    author="Andrés Vazquez",
    author_email="andres@data99.com.ar",
    description="Lista de Códigos CIE10 en español como librería python simple o para uso dentro de django como app",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cluster311/cie10",
    include_package_data=True,  # for ZIP file
    packages=['cie', 'cie10_django'],
    extras_require={
        'django':  ["Django>=1.11"],
    },
    classifiers=[
     'Programming Language :: Python :: 3',
     'Programming Language :: Python :: 3.6',
     'License :: OSI Approved :: MIT License',
     'Operating System :: OS Independent',
     'Intended Audience :: Developers',
     'Framework :: Django',
     'Framework :: Django :: 2.2',
    ],
    python_requires='>=3.6',
)
